@@ -9,12 +9,12 @@ from webob import UnicodeMultiDict
 
 import schemaish, validatish
 from formish import util
-from formish.dottedDict import dottedDict, is_int
+from dottedish import dotted, is_int
 from formish import validation
 from formish import widgets
 from formish.renderer import _default_renderer
 
-
+NOARG = object()
 
 class Action(object):
     """
@@ -45,15 +45,12 @@ def _classes(self):
     classes = [
         'field',
         re.sub('[0-9\*]+','n',_cssname(self)),
-        self.attr.__class__.__name__.lower(),
+        'type-%s'%self.attr.type.lower(),
         ]
-    if self.widget is not None:
-        classes.append(self.widget.widget.__class__.__name__.lower())
-    else:
-        classes.append('defaultwidget')
+    classes.append('widget-%s'%self.widget.type.lower())
     if self.required:
         classes.append('required')
-    if self.widget is not None and self.widget.css_class:
+    if self.widget.css_class is not None:
         classes.append(self.widget.css_class)
     if str(self.error):
         classes.append('error')
@@ -98,6 +95,12 @@ class TemplatedString(object):
             return ''
         return unicode(self.val)
 
+    def __repr__(self):
+        if not self.val:
+            return ''
+        return unicode(self.val)
+        
+
     def __nonzero__(self):
         if self.val:
             return True
@@ -105,9 +108,9 @@ class TemplatedString(object):
             return False
 
     def __call__(self):
+        widget_type, widget = self.obj.widget.template.split('.')
         renderer = self.obj.form.renderer
-        name = '%s_%s'%(self.obj.type,self.attr_name)
-        widget = self.obj.widget._template
+        name = '%s/%s'%(widget_type,self.attr_name)
         vars = {'field':self.obj}
         return fall_back_renderer(renderer, name, widget, vars)
 
@@ -125,6 +128,7 @@ class Field(object):
     
     """
     type = 'field'
+
     def __init__(self, name, attr, form):
         """
         :arg name: Name for the field
@@ -137,6 +141,9 @@ class Field(object):
         self.attr = attr
         self.form = form
 
+    def __repr__(self):
+        return 'formish.Field(name=%r, attr=%r)'% (self.name, self.attr)
+
     @property
     def title(self):
         """ The Field schema's title """
@@ -175,7 +182,7 @@ class Field(object):
     def value(self):
         """Convert the request_data to a value object for the form or None."""
         if '*' in self.name:
-            return  self.widget.pre_render(self.attr, None)
+            return  self.widget.to_request_data(self.attr, self.defaults)
         return self.form.request_data.get(self.name, None)
 
 
@@ -185,6 +192,15 @@ class Field(object):
         return validatish.validation_includes(self.attr.validator, validatish.Required)
 
 
+    @property
+    def defaults(self):
+        """Get the defaults from the form."""
+        if '*' not in self.name:
+            defaults = self.form.defaults.get(self.name, None)
+        else:
+            defaults = self.form.get_item_data(self.name, 'default', None)
+        return defaults
+
     @property
     def error(self):
         """ Lazily get the error from the form.errors when needed """
@@ -214,35 +230,28 @@ class Field(object):
 
     def __call__(self):
         """ returns a serialisation for this field using the form's renderer """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = 'field'
-        widget = self.widget._template
+        name = 'field/main'
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
             
     def label(self):
+        widget_type, widget = self.widget.template.split('.')
         """ returns the templated title """
         renderer = self.form.renderer
-        name = 'field_label'
-        widget = self.widget._template
+        name = 'field/label'
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
     
     def inputs(self):
         """ returns the templated widget """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = 'field_inputs'
-        widget = self.widget._template
+        name = 'field/inputs'
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
 
-    def __repr__(self):
-        attrclassstr = str(self.attr.__class__)
-        if attrclassstr[8:22] == 'schemaish.attr':
-            attrname = attrclassstr[23:-2]
-        else:
-            attrname = attrclassstr[8:-2]
-        return '<formish %s name="%s" attr="%s">'% (self.type, self.name, attrname)
 
 
 class CollectionFieldsWrapper(ObjectWrapper):
@@ -255,9 +264,9 @@ class CollectionFieldsWrapper(ObjectWrapper):
         self.collection = collection
 
     def __call__(self):
+        widget_type, widget = self.collection.widget.template.split('.')
         renderer = self.collection.form.renderer
-        name = '%s_fields'%self.collection.type
-        widget = self.collection.widget._template
+        name = '%s/fields'%widget_type
         vars = {'field':self.collection}
         return fall_back_renderer(renderer, name, widget, vars)
 
@@ -290,6 +299,17 @@ class Collection(object):
         if self.title is None and name is not None:
             self.title = util.title_from_name(self.name.split('.')[-1])
 
+    @property
+    def template_type(self):
+        """ Returns the template type to use for this item """
+        if self.attr.type == 'Structure':
+            name = 'structure'
+        elif self.attr.type == 'Sequence' and self.widget.type == 'SequenceDefault':
+            name = 'sequence' 
+        else:
+            name = 'field'
+        return name
+
 
     @property
     def description(self):
@@ -426,50 +446,45 @@ class Collection(object):
 
     def __call__(self):
         """ returns a serialisation for this field using the form's renderer """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = self.type
-        widget = self.widget._template
+        name = '%s/main'%widget_type
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
             
     def label(self):
         """ returns the templated title """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = '%s_label'%self.type
-        widget = self.widget._template
+        name = '%s/label'%widget_type
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
          
     def metadata(self):
         """ returns the metadata """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = '%s_metadata'%self.type
-        widget = self.widget._template
+        name = '%s/metadata'%widget_type
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
 
     def inputs(self):
         """ returns the templated widget """
+        widget_type, widget = self.widget.template.split('.')
         renderer = self.form.renderer
-        name = '%s_inputs'%self.type
-        widget = self.widget._template
+        name = '%s/inputs'%widget_type
         vars = {'field':self}
         return fall_back_renderer(renderer, name, widget, vars)
 
     def __repr__(self):
-        attrclassstr = str(self.attr.__class__)
-        if attrclassstr[8:22] == 'schemaish.attr':
-            attrname = attrclassstr[23:-2]
-        else:
-            attrname = attrclassstr[8:-2]
-        return '<formish %s name="%s" attr="%s">'% (self.type, self.name, attrname)
+        return 'formish.%s(name=%r, attr=%r)'% (self.type.title(), self.name, self.attr)
 
 class Group(Collection):
     """
     A group is a basic collection with a different template
     """
     type = 'group'
-    _template = 'structure'
+    template = 'structure'
 
 
 
@@ -478,7 +493,7 @@ class Sequence(Collection):
     A sequence is a collection with a variable number of fields depending on request data, data or min/max values
     """
     type = 'sequence'
-    _template = 'ssequence'
+    template = 'sequence'
 
     def collection_fields(self):
         """ 
@@ -539,15 +554,11 @@ class BoundWidget(object):
         setattr(self.widget, name, value)
 
     def __call__(self):
-        return self.field.form.renderer('/formish/widgets/%s/widget.html'%self._template, {'field':self.field})
+        widget_type, widget = self.widget.template.split('.')
+        return self.field.form.renderer('/formish/widgets/%s/widget.html'%widget, {'field':self.field})
 
     def __repr__(self):
-        attrclassstr = str(self.field.attr.__class__)
-        if attrclassstr[8:22] == 'schemaish.attr':
-            attrname = attrclassstr[23:-2]
-        else:
-            attrname = attrclassstr[8:-2]
-        return '<bound widget name="%s", widget="%s", type="%s">'%(self.field.name, self.widget._template, attrname)
+        return 'BoundWidget(widget=%r, field=%r)'%(self.widget, self.field)
 
 class FormFieldsWrapper(ObjectWrapper):
     """
@@ -559,7 +570,7 @@ class FormFieldsWrapper(ObjectWrapper):
         ObjectWrapper.__init__(self, form.structure.fields)
 
     def __call__(self):
-        return self.form.renderer('/formish/form_fields.html', {'form':self.form})
+        return self.form.renderer('/formish/form/fields.html', {'form':self.form})
     
 
 class Form(object):
@@ -568,7 +579,9 @@ class Form(object):
 
     The Form type is the container for all the information a form needs to
     render and validate data.
-    """    
+    """
+
+    SUPPORTED_METHODS = ['GET', 'POST']
 
     renderer = _default_renderer
 
@@ -578,7 +591,7 @@ class Form(object):
     _request_data = None
 
     def __init__(self, structure, name=None, defaults=None, errors=None,
-            action_url=None, renderer=None):
+            action_url=None, renderer=None, method='POST'):
         """
         Create a new form instance
 
@@ -599,7 +612,12 @@ class Form(object):
 
         :arg renderer: Something that returns a form serialization when called
         :type renderer: callable
+
+        :arg method: Option method, default POST
+        :type method: string
         """
+        if method.upper() not in self.SUPPORTED_METHODS:
+            raise ValueError("method must be one of GET or POST")
         # allow a single schema items to be used on a form
         if not isinstance(structure, schemaish.Structure):
             structure = schemaish.Structure([structure])
@@ -611,13 +629,25 @@ class Form(object):
         if errors is None:
             errors = {}
         self.defaults = defaults
-        self.errors = dottedDict(errors)
+        self.errors = dotted(errors)
         self.error = None
         self._actions = []
         self.action_url = action_url
         if renderer is not None:
             self.renderer = renderer
+        self.method = method
 
+    def __repr__(self):
+        attributes = []
+        attributes.append('%r'%self.structure.attr)
+        attributes.append('name=%r'%self.name)
+        if self.defaults.data != {}:
+            attributes.append('defaults=%r'%self.defaults.data)
+        if self.errors.data != {}:
+            attributes.append('errors=%r'%self.errors.data)
+        if self.action_url:
+            attributes.append('action_url=%r'%self.action_url)
+        return 'formish.Form(%s)'%( ', '.join(attributes) )
 
     def _element_name_get(self):
         """ Set the element name """
@@ -678,8 +708,9 @@ class Form(object):
         """
         if len(self._actions)==0:
             raise validation.NoActionError('The form does not have any actions')
+        request_data = getattr(request, self.method.upper())
         for action in self._actions:
-            if action.name in request.POST.keys():
+            if action.name in request_data:
                 return action.callback(request, self, *args)
         return self._actions[0].callback(request, self, *args)
 
@@ -692,11 +723,10 @@ class Form(object):
         :arg request_data: Webob style request data
         :arg raise_exceptions: Whether to raise exceptions or return errors
         """
-        data = validation.convert_request_data_to_data(self.structure, \
-                                            request_data, errors=self.errors) 
+        data = validation.from_request_data(self.structure, request_data, errors=self.errors) 
         if raise_exceptions and len(self.errors.keys()):
-            raise validation.FormError('Tried to access data but conversion' \
-        ' from request failed with %s errors (%s)'% \
+            raise validation.FormError( \
+        'Tried to access data but conversion from request failed with %s errors (%s)'% \
                    (len(self.errors.keys()), self.errors.data))
         return data
     
@@ -708,8 +738,7 @@ class Form(object):
         """
         if self._request_data is not None:
             return self._request_data
-        self._request_data = validation.convert_data_to_request_data(self.structure, \
-                                dottedDict(self.defaults))
+        self._request_data = validation.to_request_data(self.structure, dotted(self.defaults))
         return self._request_data
 
 
@@ -718,9 +747,9 @@ class Form(object):
         Assign raw request data to the form
         
         :arg request_data: raw request data (e.g. request.POST)
-        :type request_data: Dictionary (dotted or nested or dottedDict or MultiDict)
+        :type request_data: Dictionary (dotted or nested or dotted or MultiDict)
         """
-        self._request_data = dottedDict(request_data)
+        self._request_data = dotted(request_data)
 
 
     request_data = property(_get_request_data, _set_request_data)
@@ -728,7 +757,7 @@ class Form(object):
     
     def _get_defaults(self):
         """ Get the raw default data """
-        return dottedDict(self._defaults)
+        return dotted(self._defaults)
    
 
     def _set_defaults(self, data):
@@ -742,35 +771,61 @@ class Form(object):
 
     def validate(self, request, failure_callable=None, success_callable=None):
         """ 
+        Validate the form data in the request.
+
+        By default, this method returns either a dict of data or raises an
+        exception if validation fails. However, if either success_callable or
+        failure_callable are provided then the approriate callback will be
+        called, and the callback's result will be returned instead.
+
+        :arg request: the HTTP request
+        :type request: webob.Request
+        :arg failure_callable: Optional callback to call on failure.
+        :arg success_callable: Optional callback to call on success.
+        :returns: Python dict of converted and validated data.
+        :raises: formish.FormError, raised on validation failure.
+        """
+        try: 
+            data = self._validate(request)
+        except validation.FormError, e:
+            if failure_callable is None:
+                raise
+            else:
+                return failure_callable(request, self)
+        if success_callable is None:
+            return data
+        else:
+            return success_callable(request, data)
+
+    def _validate(self, request):
+        """
         Get the data without raising exceptions and then validate the data. If
         there are errors, raise them; otherwise return the data
         """
-        # If we pass in explicit failure and success callables then do this
-        # first
-        if failure_callable is not None and success_callable is not None:
-            return self._validate_and_call(request, \
-                          failure_callable, success_callable)
+        # XXX Should this happen after the basic stuff has happened?
         self.errors = {}
-        # Check this request was POSTed by this form.
-        if not request.method =='POST' and \
-           request.POST.get('__formish_form__',None) == self.name:
+        # Decide what request data to use. The method will have been validated
+        # already (unless someone's messing around!) so we can just use it in a
+        # getattr.
+        request_data = getattr(request, self.method.upper())
+        # Check this request was submitted by this form.
+        if request_data.get('__formish_form__') != self.name:
             raise Exception("request does not match form name")
-        
-        request_post = UnicodeMultiDict(request.POST, \
-                        encoding=util.get_post_charset(request))
+        # Decode request data according to the request's charset.
+        request_data = UnicodeMultiDict(request_data,
+                                        encoding=util.get_post_charset(request))
         # Remove the sequence factory data from the request
-        for k in request_post.keys():
+        for k in request_data.keys():
             if '*' in k:
-                request_post.pop(k)
+                request_data.pop(k)
         # We need the _request_data to be populated so sequences know how many
         # items they have (i.e. .fields method on a sequence uses the number of
         # values on the _request_data)
-        self._request_data = dottedDict(request_post)
-        self.request_data = validation.pre_parse_request_data( \
-                    self.structure,dottedDict(request_post))
+        self._request_data = dotted(request_data)
+        self.request_data = validation.pre_parse_incoming_request_data( \
+                    self.structure,dotted(request_data))
         data = self.get_unvalidated_data( \
                     self.request_data, raise_exceptions=False)
-        #self._request_data = dottedDict(request_post)
         try:
             self.structure.attr.validate(data)
         except schemaish.attr.Invalid, e:
@@ -778,45 +833,42 @@ class Form(object):
                 if key not in self.errors:
                     self.errors[key] = value
         if len(self.errors.keys()) > 0:
-            err_msg = 'Tried to access data but conversion' \
-                    'from request failed with %s errors'
+            err_msg = 'Tried to access data but conversion from request failed with %s errors'
             raise validation.FormError(err_msg% (len(self.errors.keys())))
         return data
 
-    def _validate_and_call(self, request, failure_callable, success_callable):
-        try: 
-            data = self.validate(request)
-        except validation.FormError, e:
-            return failure_callable(request, self)
-        return success_callable(request, data)
- 
-
-
     def set_item_data(self, key, name, value):
         """
         Allow the setting os certain attributes on item_data, a dictionary used
         to associates data with fields.
         """
-        allowed = ['title', 'widget', 'description']
+        allowed = ['title', 'widget', 'description','default']
         if name in allowed:
-            self.item_data.setdefault(key, {})[name] = value
+            if name == 'default' and '*' not in key:
+                self.defaults[key] = value
+            else:
+                self.item_data.setdefault(key, {})[name] = value
         else:
             raise KeyError('Cannot set data onto this attribute')
 
 
-    def get_item_data(self, key, name):
+    def get_item_data(self, key, name, default=NOARG):
         """
         Access item data associates with a field key and an attribute name
         (e.g. title, widget, description')
         """
-        return self.item_data.get(key, {})[name]
+        if default is NOARG:
+            data = self.item_data.get(key, {})[name]
+        else:
+            data = self.item_data.get(key, {}).get(name, default)
+        return data
 
 
     def get_item_data_values(self, name=None):
         """
         get all of the item data values
         """
-        data = dottedDict({})
+        data = dotted({})
         for key, value in self.item_data.items():
             if name is not None and value.has_key(name):
                 data[key] = value[name]
@@ -860,23 +912,25 @@ class Form(object):
         """
         Calling the Form generates a serialisation using the form's renderer
         """
-        return self.renderer('/formish/form.html', {'form':self})
+        return self.renderer('/formish/form/main.html', {'form':self})
 
     def header(self):
         """ Return just the header part of the template """
-        return self.renderer('/formish/form_header.html', {'form':self})
+        return self.renderer('/formish/form/header.html', {'form':self})
         
     def footer(self):
         """ Return just the footer part of the template """
-        return self.renderer('/formish/form_footer.html', {'form':self})
+        return self.renderer('/formish/form/footer.html', {'form':self})
         
     def metadata(self):
         """ Return just the metada part of the template """
-        return self.renderer('/formish/form_metadata.html', {'form':self})
+        return self.renderer('/formish/form/metadata.html', {'form':self})
 
     def actions(self):
         """ Return just the actions part of the template """
-        return self.renderer('/formish/form_actions.html', {'form':self})
+        return self.renderer('/formish/form/actions.html', {'form':self})
+
+        
    
 
 
