@@ -1,11 +1,11 @@
-import logging, os.path, tempfile, subprocess
+import logging, os.path, subprocess
 import formish, schemaish, validatish
-from formish.dottedDict import dottedDict
-from formish.filehandler import TempFileHandlerWeb
+from dottedish import dotted
+from formish.filestore import CachedTempFilestore
 from testish.lib import xformish
 import webob
 from urllib import urlencode
-from datetime import datetime
+import datetime
 
 log = logging.getLogger(__name__)
 
@@ -23,7 +23,7 @@ def build_request(formname, data, rawdata=False):
         for d in data:
             fields.append(d) 
     else:
-        d = dottedDict(data)
+        d = dotted(data)
         for k, v in d.dotteditems():
             fields.append( (k,v) )
     fields.append( ('submit','Submit') )
@@ -113,6 +113,15 @@ def unittest_StringDifferentEmpty(self, formdef):
     self.assertIdHasValue(f, 'form-myStringField', '8')
     self.assertRoundTrip(f, testdata)
 
+def form_Tuple(request):
+    """
+    A simple form with a single tuple field of string, integer form
+    """
+    schema = schemaish.Structure()
+    schema.add('myIntegerField', schemaish.Tuple(attrs=[schemaish.String(), schemaish.Integer()]))
+    form = formish.Form(schema, 'form')
+    return form
+
 def form_Integer(request):
     """
     A simple form with a single integer field
@@ -224,13 +233,11 @@ def form_DateDifferentEmpty(request):
     schema = schemaish.Structure()
     schema.add('myDateField', schemaish.Date())
     form = formish.Form(schema, 'form')
-    import datetime
     form['myDateField'].widget = formish.Input(empty=datetime.date.today())
     return form
 
 def unittest_DateDifferentEmpty(self, formdef):
     # Test None data
-    import datetime
     f = formdef(None)
     testdata = {'myDateField': None}
     f.defaults = testdata
@@ -363,13 +370,13 @@ def functest_Decimal(self):
     sel.type("form-myDecimalField", "1")
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal(\"1\")}"))
+    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('1')}"))
     except AssertionError, e: self.verificationErrors.append(str(e))
 
     sel.type("form-myDecimalField", "18.001")
     sel.click("form-action-submit")
     sel.wait_for_page_to_load("30000")
-    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal(\"18.001\")}"))
+    try: self.failUnless(sel.is_text_present("{'myDecimalField': Decimal('18.001')}"))
     except AssertionError, e: self.verificationErrors.append(str(e))
 
     return
@@ -416,7 +423,7 @@ def form_File(request):
     schema = schemaish.Structure()
     schema.add('myFile', schemaish.File())
     form = formish.Form(schema, 'form')
-    form['myFile'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
+    form['myFile'].widget = formish.FileUpload(filestore=CachedTempFilestore())
     return form
 
 def functest_File(self):
@@ -535,6 +542,30 @@ def form_Required(request):
     form = formish.Form(schema, 'form')
     return form
 
+def form_CheckboxRequired(request):
+    """
+    Simple Boolean Checkbox
+    """
+    schema = schemaish.Structure()
+    schema.add('checkbox', schemaish.Boolean(validator=validatish.Equal(True)))
+
+    form = formish.Form(schema, 'form')
+    form['checkbox'].widget = formish.Checkbox()
+    return form
+
+def form_RadioChoiceRequired(request):
+    """
+    A basic radio choice
+    """
+    schema = schemaish.Structure()
+    schema.add('myRadio', schemaish.Integer(validator=validatish.Required()))
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['myRadio'].widget = formish.RadioChoice(options)
+    return form
+
+
 def form_MinLength(request):
     """
     Minimum Length fields - this one is min length four chars
@@ -706,10 +737,14 @@ def form_RequiredStringAndFile(request):
     schema.add('required', schemaish.String(validator=validatish.Required()))
     schema.add('myFileField', schemaish.File())
     form = formish.Form(schema, 'form')
-    form['myFileField'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb(),show_image_preview=True,originalurl='/images/nouploadyet.png')
+    form['myFileField'].widget = formish.FileUpload(filestore=CachedTempFilestore(),show_image_thumbnail=True,image_thumbnail_default='/images/nouploadyet.png',show_download_link=True)
     return form
 
 def functest_RequiredStringAndFile(self):
+    from formish import safefilename
+    from formish import filestore
+    import shutil
+
     sel = self.selenium
     sel.open("/RequiredStringAndFile")
 
@@ -754,22 +789,35 @@ def functest_RequiredStringAndFile(self):
 
 
     filesrc = sel.get_attribute("//div[@id='actual']/img@src")
+
     filesrc = filesrc.split('/')[-1]
-    actualfilepath = 'cache/%s'%filesrc
-    assert os.path.exists(actualfilepath)
-    stdout = subprocess.Popen([IDENTIFY, actualfilepath], stdout=subprocess.PIPE).communicate()[0]
-    assert 'photo.jpg JPEG 300x300' in stdout
+
+    fs = filestore.FileSystemHeaderedFilestore(root_dir='store')
+    headers, fp = fs.get(filesrc)
+    actualfilepath = safefilename.encode(filesrc)
+    assert os.path.exists(os.path.abspath('store/%s'%actualfilepath))
+    f = open('/tmp/testish-functest','wb')
+    shutil.copyfileobj(fp, f)
+    f.close()
+
+    stdout = subprocess.Popen([IDENTIFY, '/tmp/testish-functest'], stdout=subprocess.PIPE).communicate()[0]
+    assert '/tmp/testish-functest JPEG 300x300' in stdout
     
-    actualfilepath = '%s-100x100'%actualfilepath
-    assert os.path.exists(actualfilepath)
-    stdout = subprocess.Popen([IDENTIFY, actualfilepath], stdout=subprocess.PIPE).communicate()[0]
-    assert 'photo.jpg-100x100 JPEG 100x100' in stdout
+    actualfilepath = 'store_%s-100x100'%filesrc
+    actualfilepath = safefilename.encode(actualfilepath)
+    assert os.path.exists('cache/%s'%actualfilepath)
+    fs = filestore.FileSystemHeaderedFilestore(root_dir='cache')
+    headers, fp = fs.get('store_%s-100x100'%filesrc)
+    f = open('/tmp/testish-functest','wb')
+    shutil.copyfileobj(fp, f)
+    f.close()
+    stdout = subprocess.Popen([IDENTIFY, '/tmp/testish-functest'], stdout=subprocess.PIPE).communicate()[0]
+    assert '/tmp/testish-functest JPEG 100x100' in stdout
 
           
     return
 
 
-
 ########################
 #
 #   Checkbox
@@ -1033,19 +1081,17 @@ def form_SelectChoiceDate(request):
     """
     A select choice that uses dates for values
     """
-    from datetime import date
     schema = schemaish.Structure()
     schema.add('myDateSelect', schemaish.Date())
-    options = [(date(1970,1,1),'a'),(date(1980,1,1),'b'),(date(1990,1,1),'c')]
+    options = [(datetime.date(1970,1,1),'a'),(datetime.date(1980,1,1),'b'),(datetime.date(1990,1,1),'c')]
 
     form = formish.Form(schema, 'form')
     form['myDateSelect'].widget = formish.SelectChoice(options)
     return form
 
 def unittest_SelectChoiceDate(self, formdef):
-    from datetime import date
     f = formdef(None)
-    testdata = {'myDateSelect': date(1980,1,1)}
+    testdata = {'myDateSelect': datetime.date(1980,1,1)}
     f.defaults = testdata
     request = build_request('form',testdata)
     data = f.validate(request)
@@ -1072,6 +1118,74 @@ def unittest_SelectChoiceSequenceInteger(self, formdef):
     data = f.validate(request)
     assert data == testdata
 
+#########################
+#
+#   Defaults
+#
+
+def form_SelectChoiceDefault(request):
+    """
+    A select choice that uses dates for values
+    """
+    schema = schemaish.Structure()
+    schema.add('mySelect', schemaish.Integer())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['mySelect'].widget = formish.SelectChoice(options)
+    form['mySelect'].default = 2
+    return form
+
+def form_RadioChoiceDefault(request):
+    """
+    A select choice that uses dates for values
+    """
+    schema = schemaish.Structure()
+    schema.add('myRadio', schemaish.Integer())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['myRadio'].widget = formish.RadioChoice(options)
+    form['myRadio'].default = 2
+    return form
+
+def form_CheckboxMultiChoiceDefault(request):
+    """
+    A checkbox representing a set of values
+    """
+    schema = schemaish.Structure()
+    schema.add('multiChoice', schemaish.Sequence(schemaish.Integer()))
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['multiChoice'].widget = formish.CheckboxMultiChoice(options)
+    form['multiChoice'].default = [2]
+    return form
+
+def form_SelectWithOtherChoiceDefault(request):
+    """
+    A basic select choice with input option
+    """
+    schema = schemaish.Structure()
+    schema.add('mySelect', schemaish.Integer())
+    options = [(1,'a'),(2,'b'),(3,'c')]
+
+    form = formish.Form(schema, 'form')
+    form['mySelect'].widget = formish.SelectWithOtherChoice(options)
+    form['mySelect'].default = 2
+    return form
+
+def form_SequenceOfStringsWithDefault(request):
+    """
+    A sequence with some defaults
+    """
+    schema = schemaish.Structure()
+    schema.add( 'myList', schemaish.Sequence( schemaish.String() ))
+
+    form = formish.Form(schema, 'form')
+    form.defaults = {'myList': ['a','b']}
+    return form
+
 #########################
 #
 #   Multi Select Widgets
@@ -1137,7 +1251,7 @@ def form_UploadStructure(request):
 
     form = formish.Form(schema, 'form')
 
-    form['myStruct.a'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
+    form['myStruct.a'].widget = formish.FileUpload(filestore=CachedTempFilestore())
     return form
 
 
@@ -1175,7 +1289,7 @@ def form_SequenceOfUploadStructures(request):
 
     form = formish.Form(schema, 'form')
 
-    form['myList.*.a'].widget = formish.FileUpload(filehandler=TempFileHandlerWeb())
+    form['myList.*.a'].widget = formish.FileUpload(filestore=CachedTempFilestore())
     return form
 
 def functest_SequenceOfUploadStructures(self):
@@ -1226,7 +1340,7 @@ def functest_SequenceOfStructuresWithSelects(self):
     try: self.assertEqual("b", sel.get_value("form-myList-0-b"))
     except AssertionError, e: self.verificationErrors.append(str(e))
 
-    sel.click_at("css=#form-myList-field > a", "")
+    sel.click_at("css=#form-myList--field > a", "")
     try: self.assertEqual("", sel.get_value("form-myList-1-a"))
     except AssertionError, e: self.verificationErrors.append(str(e))
 
@@ -1255,7 +1369,7 @@ def functest_SequenceOfDateParts(self):
     sel = self.selenium
     sel.open("/SequenceOfDateParts")
 
-    sel.click_at("css=#form-birthdays-field > a", "")
+    sel.click_at("css=#form-birthdays--field > a", "")
     try: self.assertEqual("", sel.get_value("form-birthdays-0-date"))
     except AssertionError, e: self.verificationErrors.append(str(e))
 
@@ -1293,6 +1407,22 @@ def form_SequenceOfStructures(request):
     return form
 
 
+def form_SequenceOfStructuresGridWidget(request):
+    """ 
+    Using a table structure to edit a sequence of dicts
+    """
+    schema = schemaish.Structure()
+    schema.add('rows', schemaish.Sequence(schemaish.Structure([
+        ('a', schemaish.Boolean()),
+        ('b', schemaish.String()),])))
+
+    form = formish.Form(schema, 'form')
+    form['rows'].widget = formish.Grid()
+    form['rows.*.a'].widget = formish.Checkbox()
+    form.defaults = {'rows': [{'a':True,'b':'2'},{'a':False,'b':'4'},{'a':False,'b':'6'}]}
+    return form
+
+
 def form_GranularFormLayout(request):
     """
     A simple demonstration of partial rendering of parts of forms.
@@ -1423,7 +1553,7 @@ ${form.metadata()|n}
 
 ${form['firstName']()|n}
 
-<div id="${form['surname'].cssname}-field" class="${form['surname'].classes}">
+<div id="${form['surname'].cssname}--field" class="${form['surname'].classes}">
   <strong>${form['surname'].description}</strong>
   <em>${form['surname'].error}</em>
   ${form['surname'].widget()|n}
