@@ -3,12 +3,11 @@ import unittest
 import webob
 import urllib
 from BeautifulSoup import BeautifulSoup
-from formish.dottedDict import dottedDict
-import webob
+from dottedish import dotted
 from urllib import urlencode
 
 def build_request(formname, data):
-    d = dottedDict(data)
+    d = dotted(data)
     e = {'REQUEST_METHOD': 'POST'}
     request = webob.Request.blank('/',environ=e)
     fields = []
@@ -27,7 +26,7 @@ class Test(unittest.TestCase):
         r = webob.Request.blank('http://localhost/')
         r.method = 'POST'
         r.content_type = 'application/x-www-form-urlencoded'
-        kvpairs = []
+        kvpairs = [('__formish_form__', 'form')]
         for k in d.dottedkeys():
             lastsegment = k.split('.')[-1]
             try:
