@@ -1,7 +1,7 @@
 import formish
 import unittest
 import schemaish
-from formish.dottedDict import dottedDict
+from dottedish import dotted
 from formish.forms import validation
 import copy
 from webob import MultiDict
@@ -15,12 +15,14 @@ class DummyObject(object):
 class Request(object):
     headers = {'content-type':'text/html'}
     
-    def __init__(self, form_name='form', POST=None):
-        if POST is None:
-            POST = {}
-        self.POST = MultiDict(POST)
-        self.POST['__formish_form__'] = form_name
-        self.method = 'POST'
+    def __init__(self, form_name='form', POST=None, GET=None, method='POST'):
+        # Build GET and POST data
+        self.GET = MultiDict(GET or {})
+        self.POST = MultiDict(POST or {})
+        # Add the form's name to the appropriate request data dict.
+        getattr(self, method.upper())['__formish_form__'] = form_name
+        # Set the method
+        self.method = method
         
             
 class TestFormBuilding(unittest.TestCase):
@@ -91,10 +93,10 @@ class TestFormBuilding(unittest.TestCase):
         request =  Request(name, r)
         form = formish.Form(schema_nested, name)
         # request to data
-        rdtd = validation.convert_request_data_to_data(form.structure, dottedDict(copy.deepcopy(request.POST)))
-        assert rdtd == dottedDict(reqr)
+        rdtd = validation.from_request_data(form.structure, dotted(copy.deepcopy(request.POST)))
+        assert rdtd == dotted(reqr)
         # data to request
-        dtrd = validation.convert_data_to_request_data(form.structure, dottedDict(data))
+        dtrd = validation.to_request_data(form.structure, dotted(data))
         assert dtrd == reqrdata
 
 
@@ -153,15 +155,9 @@ class TestFormBuilding(unittest.TestCase):
         # Does the form produce an int and a string
         self.assertEquals(form.validate(request), {'a': 3, 'b': '4'})
         # Does the convert request to data work
-        self.assertEqual( validation.convert_request_data_to_data(form.structure, dottedDict(request.POST)) , {'a': 3, 'b': '4'})
+        self.assertEqual( validation.from_request_data(form.structure, dotted(request.POST)) , {'a': 3, 'b': '4'})
         # Does the convert data to request work
-        self.assert_( validation.convert_data_to_request_data(form.structure, dottedDict( {'a': 3, 'b': '4'} )) == reqr)
-
-def success(request, data):
-    return 'success'
-
-def failure(request, form):
-    return 'failure'
+        self.assert_( validation.to_request_data(form.structure, dotted( {'a': 3, 'b': '4'} )) == reqr)
 
     def test_failure_and_success_callables(self):
 
@@ -172,12 +168,16 @@ def failure(request, form):
 
         r = {'a':'2','b': '4'}
         request = Request(name, r)
-        self.assertEquals(form.validate(request, success, failure), 'failure')
+        self.assertEquals(form.validate(request, failure, success), 'failure')
+        self.assertEquals(form.validate(request, failure), 'failure')
+        self.assertRaises(formish.FormError, form.validate, request, success_callable=success)
 
         r = {'a': '12', 'b': '4'}
         request = Request(name, r)
         form = formish.Form(schema_flat, name)
-        self.assertEquals(form.validate(request, success, failure), 'success')
+        self.assertEquals(form.validate(request, failure, success), 'success')
+        self.assertEquals(form.validate(request, success_callable=success), 'success')
+        self.assertEquals(form.validate(request, failure_callable=failure), {'a': 12, 'b': '4'})
         
           
     def test_datetuple_type(self):
@@ -196,14 +196,14 @@ def failure(request, form):
         # Check the data is converted correctly
         self.assertEquals(form.validate(request), {'a': d, 'b': '4'})
         # Check req to data
-        self.assertEqual( validation.convert_request_data_to_data(form.structure, dottedDict(request.POST)) , dottedDict({'a': d, 'b': '4'}))
+        self.assertEqual( validation.from_request_data(form.structure, dotted(request.POST)) , dotted({'a': d, 'b': '4'}))
         # Check data to req
-        self.assert_( validation.convert_data_to_request_data(form.structure, dottedDict( {'a': d, 'b': '4'} )) == dottedDict({'a': {'month': [3], 'day': [1], 'year': [1966]}, 'b': ['4']}))
+        self.assert_( validation.to_request_data(form.structure, dotted( {'a': d, 'b': '4'} )) == dotted({'a': {'month': [3], 'day': [1], 'year': [1966]}, 'b': ['4']}))
 
     def test_form_retains_request_data(self):
         form = formish.Form(schemaish.Structure([("field", schemaish.String())]))
         assert 'name="field" value=""' in form()
-        data = form.validate(Request('formish', {'field': 'value'}))
+        data = form.validate(Request('form', {'field': 'value'}))
         assert data == {'field': 'value'}
         assert form.request_data['field'] == ['value']
         assert 'name="field" value="value"' in form()
@@ -229,6 +229,62 @@ def failure(request, form):
         assert form.request_data == {'field': ['default value']}
         assert 'name="field" value="default value"' in form()
 
+    def test_method(self):
+        schema = schemaish.Structure([('string', schemaish.String())])
+        # Default should be POST
+        self.assertTrue('method="post"' in formish.Form(schema)().lower())
+        # (Crudely) check that an explicit method is rendered correctly by the
+        # templates.
+        for method in ['POST', 'GET', 'get', 'Get']:
+            expected = ('method="%s"' % method).lower()
+            rendered = formish.Form(schema, method=method)().lower()
+            self.assertTrue(expected in rendered)
+        # Check that unsupported methods are rejected.
+        self.assertRaises(ValueError, formish.Form, schema, method='unsupported')
+        # Check that default (POST) and non-default (e.g. GET) forms validate.
+        for method, request in [('post', Request(POST={'string': 'abc'})),
+                                ('get', Request(GET={'string': 'abc'}, method='GET'))]:
+            data = formish.Form(schema, method=method).validate(request)
+            self.assertTrue(data == {'string': 'abc'})
+
+
+class TestActions(unittest.TestCase):
+
+    def test_get(self):
+        request = Request(method='GET')
+        self.assertTrue(self._form('GET').action(request) == 'one')
+        request = Request(GET={'one': 'One'}, method='GET')
+        self.assertTrue(self._form('GET').action(request) == 'one')
+        request = Request(GET={'two': 'Two'}, method='GET')
+        self.assertTrue(self._form('GET').action(request) == 'two')
+
+    def test_post(self):
+        request = Request(method='POST')
+        self.assertTrue(self._form('POST').action(request) == 'one')
+        request = Request(POST={'one': 'One'}, method='POST')
+        self.assertTrue(self._form('POST').action(request) == 'one')
+        request = Request(POST={'two': 'Two'}, method='POST')
+        self.assertTrue(self._form('POST').action(request) == 'two')
+
+    def _form(self, method):
+        def callback1(*a, **k):
+            return 'one'
+        def callback2(*a, **k):
+            return 'two'
+        schema = schemaish.Structure([('string', schemaish.String())])
+        form = formish.Form(schema, method=method)
+        form.add_action(callback1, 'one')
+        form.add_action(callback2, 'two')
+        return form
+
+
+def success(request, data):
+    return 'success'
+
+
+def failure(request, form):
+    return 'failure'
+
 
 class TestBugs(unittest.TestCase):
 
