@@ -4,7 +4,7 @@ calls the converters that do so) and also converts dotted numeric formats into
 sequences (e.g. a.0 and a.1 onto a[0] and a[1]). It also includes some
 validation exceptions.
 """
-from formish.dottedDict import dottedDict
+from dottedish import dotted
 from validatish import Invalid
 from convertish import convert
 
@@ -22,7 +22,7 @@ def convert_sequences(data):
     try:
         int(data.keys()[0])
     except ValueError:
-        return dottedDict(data)
+        return dotted(data)
     # collect the keys as sorted integers
     intkeys = []
     for key in data.keys():
@@ -90,8 +90,7 @@ def getNestedProperty(data, dottedkey):
         return None
 
 
-def convert_data_to_request_data(form_structure, data,
-                      request_data=None, errors=None):
+def to_request_data(form_structure, data, request_data=None, errors=None):
     """
     Take a form structure and use it's widgets to convert schema data (dict) to
     request data
@@ -105,28 +104,27 @@ def convert_data_to_request_data(form_structure, data,
     
     """
     if request_data is None:
-        request_data = dottedDict()
+        request_data = dotted()
     if errors is None:
-        errors = dottedDict()
+        errors = dotted()
     for field in form_structure.fields:
         try:
             if field.type is 'group' or \
               (field.type is 'sequence' and \
                 (field.widget is None or \
                   field.widget.converttostring is False)):
-                convert_data_to_request_data(field, data,
+                to_request_data(field, data,
                         request_data=request_data, errors=errors)
             else:
                 item_data = getNestedProperty(data, field.name)
                 request_data[field.name] = \
-                        field.widget.pre_render(field.attr, item_data)
+                        field.widget.to_request_data(field.attr, item_data)
         except Invalid, e:
             errors[field.name] = e
             raise
     return request_data
         
-def convert_request_data_to_data(form_structure,
-                  request_data, data=None, errors=None):
+def from_request_data(form_structure, request_data, data=None, errors=None):
     """
     Take a form structure and use it's widgets to convert request data to schema data (dict)
     
@@ -152,18 +150,18 @@ def convert_request_data_to_data(form_structure,
                     # this and there are no items in the list then this key
                     # wouldn't appear.
                     data[field.name] = []
-                convert_request_data_to_data(field, request_data,
+                from_request_data(field, request_data,
                                           data=data, errors=errors)
             else: 
-                data[field.name] = field.widget.convert(field.attr, \
+                data[field.name] = field.widget.from_request_data(field.attr, \
                                                 request_data.get(field.name,[]))
         except convert.ConvertError, e:
             errors[field.name] = e.message
     
-    data = recursive_convert_sequences(dottedDict(data))
+    data = recursive_convert_sequences(dotted(data))
     return data
 
-def pre_parse_request_data(form_structure, request_data, data=None):
+def pre_parse_incoming_request_data(form_structure, request_data, data=None):
     """
     Some widgets (at the moment only files) need to have their data massaged in
     order to make sure that data->request and request->data is symmetric
@@ -176,13 +174,13 @@ def pre_parse_request_data(form_structure, request_data, data=None):
         if field.type is 'group' or \
           (field.type == 'sequence' and \
              field.widget._template is 'SequenceDefault'):
-            pre_parse_request_data(field, request_data, data=data)
+            pre_parse_incoming_request_data(field, request_data, data=data)
         else: 
             # This needs to be cleverer...
             item_data = request_data.get(field.name, [])
-            data[field.name] = field.widget.pre_parse_request(field.attr, item_data, full_request_data=request_data)
+            data[field.name] = field.widget.pre_parse_incoming_request_data(field.attr, item_data, full_request_data=request_data)
 
-    return dottedDict(data)
+    return dotted(data)
 
 
 class FormishError(Exception):
