@@ -4,12 +4,13 @@ Commonly needed form widgets.
 
 __all__ = ['Input', 'Password', 'CheckedPassword', 'Hidden', 'TextArea',
         'Checkbox', 'DateParts', 'FileUpload', 'SelectChoice','SelectWithOtherChoice','RadioChoice',
-        'CheckboxMultiChoice', 'SequenceDefault','CheckboxMultiChoiceTree']
+        'CheckboxMultiChoice', 'SequenceDefault','CheckboxMultiChoiceTree', 'Grid']
 
 from convertish.convert import string_converter, \
         datetuple_converter,ConvertError
 from schemaish.type import File as SchemaFile
-from formish import dottedDict
+from dottedish import get_dict_from_dotted_dict
+import uuid
 
 
 UNSET = object()
@@ -19,19 +20,20 @@ class Widget(object):
     """
     Base class for widgets
     """
-
-    _template = None
+    
+    type = None
+    template = None
     
     def __init__(self, **k):
-        self.converter_options = k.get('converter_options', {})
         self.css_class = k.get('css_class', None)
         self.converttostring = True
         self.empty = k.get('empty', None)
+        self.converter_options = k.get('converter_options', {})
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = ','
     
 
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         Before the widget is rendered, the data is converted to a string
         format.If the data is None then we return an empty string. The sequence
@@ -43,7 +45,7 @@ class Widget(object):
         return [string_data]
 
 
-    def pre_parse_request(self, schema_type, request_data, full_request_data):
+    def pre_parse_incoming_request_data(self, schema_type, request_data, full_request_data):
         """
         Prior to convert being run, we have a chance to munge the data. This is
         only used by file upload at the moment
@@ -51,7 +53,7 @@ class Widget(object):
         return request_data
 
 
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         after the form has been submitted, the request data is converted into
         to the schema type.
@@ -61,12 +63,21 @@ class Widget(object):
             return self.empty
         return string_converter(schema_type).to_type(string_data)
 
-    def __repr__(self):
-        return '<widget "%s">'% (self._template)
+
 
     def __call__(self, field):
-        return field.form.renderer('/formish/widgets/%s.html'%self._template, {'f':field})
+        return field.form.renderer('/formish/widgets/%s.html'%self.template, {'f':field})
 
+    def __repr__(self):
+        attributes = []
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
 
 
 
@@ -75,7 +86,8 @@ class Input(Widget):
     Basic input widget type, used for text input
     """
 
-    _template = 'Input'
+    type = 'Input'
+    template = 'field.Input'
 
     def __init__(self, **k):
         self.strip = k.pop('strip', True)
@@ -83,7 +95,7 @@ class Input(Widget):
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = ','
 
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Default to stripping whitespace
         """
@@ -94,13 +106,26 @@ class Input(Widget):
             return self.empty
         return string_converter(schema_type).to_type(string_data)
 
+    def __repr__(self):
+        attributes = []
+        if self.strip is False:
+            attributes.append('strip=%r'%self.strip)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
 
 
 class Password(Input):
     """
     Password widget is a basic input type but using password html input type
     """
-    _template = 'Password'
+    type = 'Password'
+    template = 'field.Password'
 
 
    
@@ -109,7 +134,8 @@ class CheckedPassword(Input):
     Checked Password ensures that the password has been entered twice
     """
 
-    _template = 'CheckedPassword'
+    type = 'CheckedPassword'
+    template = 'field.CheckedPassword'
 
     def __init__(self, **k):
         self.strip = k.pop('strip', True)
@@ -118,7 +144,7 @@ class CheckedPassword(Input):
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = ','
             
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         Extract both the password and confirm fields
         """
@@ -127,7 +153,7 @@ class CheckedPassword(Input):
             return {'password': [''], 'confirm': ['']}
         return {'password': [string_data], 'confirm': [string_data]}
     
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Check the password and confirm match (when stripped)
         """
@@ -142,13 +168,26 @@ class CheckedPassword(Input):
             return self.empty
         return string_converter(schema_type).to_type(password)
 
+    def __repr__(self):
+        attributes = []
+        if self.strip is False:
+            attributes.append('strip=%r'%self.strip)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
 
 
 class Hidden(Input):
     """
     Basic input but using a hidden html input field
     """
-    _template = 'Hidden'
+    type = 'Hidden'
+    template = 'field.Hidden'
 
 
 
@@ -162,7 +201,8 @@ class SequenceDefault(Widget):
         activated)
     """
 
-    _template = 'SequenceDefault'
+    type = 'SequenceDefault'
+    template = 'sequence.-'
 
     def __init__(self, **k):
         Widget.__init__(self, **k)
@@ -172,12 +212,30 @@ class SequenceDefault(Widget):
         self.sortable = k.get('sortable', True)
         self.converttostring = False
 
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
-        Short circuits the usual pre_render
+        Short circuits the usual to_request_data
         """
         return data
 
+    def __repr__(self):
+        attributes = []
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+        if self.min:
+            attributes.append('min=%r'%self.min)
+        if self.max:
+            attributes.append('max=%r'%self.max)
+        if self.addremove:
+            attributes.append('addremove=%r'%self.addremove)
+        if self.sortable:
+            attributes.append('sortable=%r'%self.sortable)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
         
 class TextArea(Input):
     """
@@ -187,10 +245,10 @@ class TextArea(Input):
     :arg rows: set the cols attr on the textarea element
     """
 
-    _template = 'TextArea'
+    type = 'TextArea'
+    template = 'field.TextArea'
     
     def __init__(self, **k):
-        Input.__init__(self, **k)
         self.cols = k.pop('cols', None)
         self.rows = k.pop('rows', None)
         self.strip = k.pop('strip', True)
@@ -198,7 +256,7 @@ class TextArea(Input):
         if not self.converter_options.has_key('delimiter'):
             self.converter_options['delimiter'] = '\n'
     
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         We're using the converter options to allow processing sequence data
         using the csv module
@@ -209,7 +267,7 @@ class TextArea(Input):
             return ['']
         return [string_data]
     
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         We're using the converter options to allow processing sequence data
         using the csv module
@@ -222,15 +280,61 @@ class TextArea(Input):
         return string_converter(schema_type).to_type(string_data,
             converter_options=self.converter_options)
 
+    def __repr__(self):
+        attributes = []
+        if self.strip is False:
+            attributes.append('strip=%r'%self.strip)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
+class Grid(Input):
+    """
+    Grid input field
+    """
+
+    type = 'Grid'
+    template = 'field.Grid'
+    
+    def __init__(self, **k):
+        Input.__init__(self, **k)
+        if not self.converter_options.has_key('delimiter'):
+            self.converter_options['delimiter'] = '\n'
+        self.converttostring = False
+    
+    def to_request_data(self, schema_type, data):
+        string_data = string_converter(schema_type).from_type(data, \
+            converter_options=self.converter_options)
+        return [string_data]
+    
+    def from_request_data(self, schema_type, request_data):
+        string_data = request_data[0]
+        return string_converter(schema_type).to_type(string_data,
+            converter_options=self.converter_options)
+
+    def __repr__(self):
+        attributes = []
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
     
 class Checkbox(Widget):
     """
     Checkbox widget, defaults to True or False
     """
 
-    _template = 'Checkbox'
+    type = 'Checkbox'
+    template = 'field.Checkbox'
 
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         If the request data exists, then we treat this as True
         """
@@ -246,7 +350,8 @@ class DateParts(Widget):
     Simple three part date entry form
     """
 
-    _template = 'DateParts'
+    type = 'DateParts'
+    template = 'field.DateParts'
     
     def __init__(self, **k):
         self.strip = k.pop('strip', True)
@@ -256,7 +361,7 @@ class DateParts(Widget):
             self.converter_options['delimiter'] = ','
 
         
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         Convert to date parts
         """
@@ -267,7 +372,7 @@ class DateParts(Widget):
                 'month': [dateparts[1]],
                 'day': [dateparts[2]]}
     
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Pull out the parts and convert
         """
@@ -279,6 +384,21 @@ class DateParts(Widget):
         else:
             return self.empty
         return datetuple_converter(schema_type).to_type(date_parts)
+
+    def __repr__(self):
+        attributes = []
+        if self.strip is False:
+            attributes.append('strip=%r'%self.strip)
+        if self.day_first is not None:
+            attributes.append('day_first=%r'%self.day_first)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
         
 
 class FileUpload(Widget):
@@ -286,66 +406,109 @@ class FileUpload(Widget):
     File upload widget.
     """
 
-    _template = 'FileUpload'
+    type = 'FileUpload'
+    template = 'field.FileUpload'
     
-    def __init__(self, filehandler, show_image_preview=False, \
-                 allow_clear=True, css_class=None, originalurl=None):
+    def __init__(self, filestore, show_file_preview=True, show_download_link=False, \
+                 show_image_thumbnail=False, url_base=None, \
+                 css_class=None, image_thumbnail_default=None, url_ident_factory=None):
         """
-        :arg filehandler: filehandler is any object with the following methods:
+        :arg filestore: filestore is any object with the following methods:
 
             storeFile(self, f)
                 where f is a file instance
 
-            getUrlForFile(self, data)
-                where data is the form item data or a path to a temporary file
-                and is expected to return a URL to access the persisted or
-                temporary data.
-
-        :arg show_image_preview: a boolean that, if set, will include an image
+        :arg show_image_thumbnail: a boolean that, if set, will include an image
             thumbnail with the widget
         :arg css_class: extra css classes to apply to the widget
-        :arg originalurl: a default url to 
+        :arg image_thumbnail_default: a default url to 
+        XXX image_thumbnail_default -> default_image 
+        XXX allow_clear -> allow_delete 
+        XXX url_ident_factory -> filestore_key_factory
         """
         Widget.__init__(self)
-        self.css_class = css_class
-        self.filehandler = filehandler
-        self.show_image_preview = show_image_preview
-        self.allow_clear = allow_clear
-        self.originalurl = originalurl
+        self.filestore = filestore
+        self.show_image_thumbnail = show_image_thumbnail
+        self.image_thumbnail_default = image_thumbnail_default
+        if url_base is None:
+            self.url_base = '/filehandler'
+        else:
+            self.url_base = url_base
+        self.show_download_link = show_download_link
+        self.show_file_preview = show_file_preview
+        if url_ident_factory is not None:
+            self.url_ident_factory = url_ident_factory
+        else:
+            self.url_ident_factory = lambda i: i.filename
+
+    def __repr__(self):
+        attributes = []
+        attributes.append('filestore=%r'%self.filestore)
+        if self.show_image_thumbnail == True:
+            attributes.append('show_image_thumbnail=True')
+        if self.image_thumbnail_default:
+            attributes.append('image_thumbnail_default=%r'%self.image_thumbnail_default)
+        if self.url_base != "/filehandler":
+            attributes.append('url_base=%r'%self.url_base)
+        if self.show_download_link == True:
+            attributes.append('show_download_link=True')
+        if self.show_file_preview == False:
+            attributes.append('show_file_preview=False')
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+          
+
+    def urlfactory(self, data):
+        if not data:
+            return self.image_thumbnail_default
+        if isinstance(data, SchemaFile):
+            key = self.url_ident_factory(data)
+        else:
+            key = data
+        return '%s/%s' % (self.url_base, key)
     
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         We use the url factory to get an identifier for the file which we use
         as the name. We also store it in the 'default' field so we can check if
         something has been uploaded (the identifier doesn't match the name)
         """
+        mimetype = ''
         if isinstance(data, SchemaFile):
-            self.default = self.filehandler.urlfactory(data)
+            default = self.url_ident_factory(data)
+            mimetype = data.mimetype
         elif data is not None:
-            self.default = data
+            default = data
         else:
-            self.default = ''
-        return {'name': [self.default], 'default':[self.default]}
+            default = ''
+        return {'name': [default], 'default':[default], 'mimetype':[mimetype]}
     
-    def pre_parse_request(self, schema_type, data, full_request_data):
+    def pre_parse_incoming_request_data(self, schema_type, data, full_request_data):
         """
         File uploads are wierd; in out case this means assymetric. We store the
         file in a temporary location and just store an identifier in the field.
         This at least makes the file look symmetric.
         """
         if data.get('remove', [None])[0] is not None:
-            # Removing the file
             data['name'] = ['']
+            data['mimetype'] = ['']
             return data
 
         fieldstorage = data.get('file', [''])[0]
-        if fieldstorage is not u'':
-            # Storing an uploaded file
-            name = self.filehandler.store_file(fieldstorage)
-            data['name'] = [name]
+        if getattr(fieldstorage,'file',None):
+            filename = '%s-%s'%(uuid.uuid4().hex,fieldstorage.filename)
+            self.filestore.put(filename, fieldstorage.file, fieldstorage.type, uuid.uuid4().hex)
+            data['name'] = [filename]
+            data['mimetype'] = [fieldstorage.type]
         return data
     
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Creates a File object if possible
         """
@@ -357,13 +520,11 @@ class FileUpload(Widget):
             return SchemaFile(None, None, None)
         else:
             filename = request_data['name'][0]
-            path_for_file = self.filehandler.get_path_for_file(filename)
-            filepath = open(path_for_file)
-            mimetype = self.filehandler.get_mimetype(filename)
-            filetype = SchemaFile(filepath, filename, mimetype)
-            return filetype
-
-
+            try:
+                content_type, cache_tag, f = self.filestore.get(filename)
+            except KeyError:
+                return None
+            return SchemaFile(f, filename, content_type)
 
     
 class SelectChoice(Widget):
@@ -371,7 +532,8 @@ class SelectChoice(Widget):
     Html Select element
     """
 
-    _template = 'SelectChoice'
+    type = 'SelectChoice'
+    template = 'field.SelectChoice'
 
     none_option = (None, '- choose -')
 
@@ -386,6 +548,7 @@ class SelectChoice(Widget):
             self.none_option = none_option
         Widget.__init__(self, **k)
         self.options = _normalise_options(options)
+
             
     def selected(self, option, value, schema_type):
         """
@@ -395,11 +558,7 @@ class SelectChoice(Widget):
             v = self.empty
         else:
             v = value
-        try:
-            cv = string_converter(schema_type).to_type(v)
-        except ConvertError:
-            return ''
-        if option[0] == cv:
+        if option[0] == v:
             return ' selected="selected"'
         else:
             return ''
@@ -424,12 +583,27 @@ class SelectChoice(Widget):
         if none_option is self.empty:
             return ''
         return none_option
+
+    def __repr__(self):
+        attributes = []
+        attributes.append('options=%s'%repr(self.options))
+        if self.none_option is not UNSET:
+            attributes.append('none_option=%s'%repr(self.none_option))
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
     
 class SelectWithOtherChoice(SelectChoice):
     """
     Html Select element
     """
-    _template = 'SelectWithOtherChoice'
+    type = 'SelectWithOtherChoice'
+    template = 'field.SelectWithOtherChoice'
 
     other_option = ('...', 'Other ...')
 
@@ -440,7 +614,7 @@ class SelectWithOtherChoice(SelectChoice):
         self.strip = k.pop('strip',True)
         SelectChoice.__init__(self, options, **k)
 
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         populate the other choice if needed
         """
@@ -449,14 +623,12 @@ class SelectWithOtherChoice(SelectChoice):
             return {'select': ['...'], 'other': [string_data]}
         return {'select': [string_data], 'other': ['']}
 
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Check to see if we need to use the 'other' value
         """
         select = request_data['select'][0]
         other = request_data['other'][0]
-        if self.strip:
-            other = other.strip()
         if select == '...':
             value = other
         else:
@@ -480,34 +652,50 @@ class SelectWithOtherChoice(SelectChoice):
             v = self.empty
         else:
             v = value
-        # Convert or raise
-        try:
-            cv = string_converter(schema_type).to_type(v)
-        except ConvertError:
-            return ''
         # Check for selected
-        if option[0] == cv:
+        if option[0] == v:
             return ' selected="selected"'
         else:
             return ''
 
-class RadioChoice(Widget):
+    def __repr__(self):
+        attributes = []
+        attributes.append('options=%r'%self.options)
+        if self.none_option is not UNSET:
+            attributes.append('none_option=%r'%list(self.none_option))
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
+class RadioChoice(SelectChoice):
     """
     Radio choice html element
     """
 
-    _template = 'RadioChoice'
+    type = 'RadioChoice'
+    template = 'field.RadioChoice'
 
     none_option = (None, '- choose -')
 
-    def __init__(self, options, **k):
-        none_option = k.pop('none_option', UNSET)
-        if none_option is not UNSET:
-            self.none_option = none_option
-        Widget.__init__(self, **k)
-        self.options = _normalise_options(options)
-            
-    def convert(self, schema_type, request_data):
+    def selected(self, option, value, schema_type):
+        """
+        Check if the currently rendering input is the same as the value
+        """
+        if value == '':
+            v = self.empty
+        else:
+            v = value
+        if option[0] == v:
+            return ' checked="checked"'
+        else:
+            return ''
+
+    def from_request_data(self, schema_type, request_data):
         """
         If we don't have a choice, set a blank value
         """
@@ -521,23 +709,6 @@ class RadioChoice(Widget):
             return self.empty
 
         return string_converter(schema_type).to_type(string_data)
-
-    def selected(self, option, value, schema_type):
-        """
-        Check if the currently rendering input is the same as the value
-        """
-        if value == '':
-            v = self.empty
-        else:
-            v = value
-        try:
-            cv = string_converter(schema_type).to_type(v)
-        except ConvertError:
-            return ''
-        if option[0] == cv:
-            return ' checked="checked"'
-        else:
-            return ''
     
     def get_none_option_value(self, schema_type):
         """
@@ -547,19 +718,34 @@ class RadioChoice(Widget):
         if none_option is self.empty:
             return ''
         return none_option
+
+    def __repr__(self):
+        attributes = []
+        attributes.append('options=%r'%self.options)
+        if self.none_option is not UNSET:
+            attributes.append('none_option=%r'%list(self.none_option))
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
     
 class CheckboxMultiChoice(Widget):
     """
     Checkbox multi choice is a set of checkboxes that for a sequence of data
     """
 
-    _template = 'CheckboxMultiChoice'
+    type = 'CheckboxMultiChoice'
+    template = 'field.CheckboxMultiChoice'
 
     def __init__(self, options, css_class=None):
         self.options = _normalise_options(options)
         Widget.__init__(self, css_class=css_class)
             
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         """
         Iterate over the data, converting each one
         """
@@ -567,7 +753,7 @@ class CheckboxMultiChoice(Widget):
             return []
         return [string_converter(schema_type.attr).from_type(d) for d in data]
     
-    def convert(self, schema_type, request_data):
+    def from_request_data(self, schema_type, request_data):
         """
         Iterating to convert back to the source data
         """
@@ -591,33 +777,79 @@ class CheckboxMultiChoice(Widget):
         else:
             return ''
 
+    def __repr__(self):
+        attributes = []
+        attributes.append('options=%r'%self.options)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
+
+def get_parent(segments):
+    if len(segments) == 1:
+        return ''
+    else:
+        return '.'.join(segments[:-1])
+
+def mktree(options):
+    last_segments_len = 1
+    root = {'': {'data':('root', 'Root'), 'children':[]} }
+    for id, label in options:
+        segments = id.split('.')
+        parent = get_parent(segments)
+        root[id] = {'data': (id, label), 'children':[]}
+        root[parent]['children'].append(root[id])
+    return root['']
+
+
 class CheckboxMultiChoiceTree(Widget):
     """
     A more complicated checkbox select that
     """
 
-    _template = 'CheckboxMultiChoiceTree'
+    type = 'CheckboxMultiChoiceTree'
+    template = 'field.CheckboxMultiChoiceTree'
 
     def __init__(self, options, cssClass=None):
         self.options = options
-        self.optiontree = dottedDict._getDictFromDottedKeyDict(dict(options),noexcept=True) 
+        self.optiontree = mktree(options)
         Widget.__init__(self,cssClass=cssClass)
             
-    def pre_render(self, schema_type, data):
+    def to_request_data(self, schema_type, data):
         if data is None: 
             return []
         return [string_converter(schema_type.attr).from_type(d) for d in data]
     
-    def convert(self, schema_type, data):
+    def from_request_data(self, schema_type, data):
         return [string_converter(schema_type.attr).to_type(d) for d in data]
 
     def checked(self, option, values, schema_type):
         if values is not None:
-            typed_values = self.convert(schema_type,values)
+            typed_values = self.from_request_data(schema_type,values)
         if values is not None and option[0] in typed_values:
             return ' checked="checked"'
         else:
             return ''        
+
+    def __repr__(self):
+        attributes = []
+        attributes.append('options=%r'%self.options)
+        if self.none_option is not UNSET:
+            attributes.append('none_option=%r'%self.none_option)
+        if self.strip is False:
+            attributes.append('strip=%s'%self.strip)
+        if self.converter_options != {'delimiter':','}:
+            attributes.append('converter_options=%r'%self.converter_options)
+        if self.css_class:
+            attributes.append('css_class=%r'%self.css_class)
+        if self.empty is not None:
+            attributes.append('empty=%r'%self.empty)
+
+        return 'formish.%s(%s)'%(self.__class__.__name__, ', '.join(attributes))
         
 def _normalise_options(options):
     """
